/*---------------------------------------------------------------------------*/

/*!  \file  frisca_cg_gpu.c
     \brief  a simple CG slip-spring rheology simulator on a GPU.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2009-2011 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include "frisca_cg_gpu.h"

/*---------------------------------------------------------------------------*/
int main(int argc, char **argv)
{
    int iterations = 100;
    int p = 2000;
    int n = 8;
    int m = 512;
    float n0 = 1.0f;
    float n_s = 0.5f;
    float zeta_s = 0.1f;
    float dt = 0.01f;
    float kappa[3][3] = {{0.0f, 0.5f, 0.0f},
                         {0.0f, 0.0f, 0.0f},
                         {0.0f, 0.0f, 0.0f}};
    unsigned long seed = 1239872;

    simulator_state *state;
    int i;


        /*  initialization  */

    initialize_device(argc,argv);

    state  = allocate_simulator_state(m,n);

    initialize_simulator_state(state,seed,n0,n_s);


    set_simulation_parameters(state,n0,n_s,zeta_s,dt,kappa,iterations);

    copy_simulator_state_host_to_device(state);

        calculate_stress_tensor(state);

        printf("%f %f %f %f %f %f\n",
               state->sigma[0],state->sigma[1],state->sigma[2],
               state->sigma[3],state->sigma[4],state->sigma[5]);

    for(i = 0;i < p;i++)
    {
        perform_simulation(state);
        calculate_stress_tensor(state);

        printf("%f %f %f %f %f %f\n",
               state->sigma[0],state->sigma[1],state->sigma[2],
               state->sigma[3],state->sigma[4],state->sigma[5]);
    }


    copy_simulator_state_device_to_host(state);

    store_simulator_state_to_file("frisca_cg_gpu_state.h5",state);

        /*  finalization  */

    free_simulator_state(state);

    finalize_device();

    return 0;
}

/*---------------------------------------------------------------------------*/
