/*---------------------------------------------------------------------------*/

/*!  \file  rng_gpu.h
     \brief  pseudo random number generator on GPU  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2009,2010 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#ifndef INCLUDE_RNG_GPU_H
#define INCLUDE_RNG_GPU_H

/*---------------------------------------------------------------------------*/
#define RAND_GPU_MAX 0x7fffffffU

#define SRAND_GPU_A 69069U
#define SRAND_GPU_C 65539U

#define LCG_GPU_A 1664525U
#define LCG_GPU_C 1U
#define XORSHIFT_GPU_A 5
#define XORSHIFT_GPU_B 17
#define XORSHIFT_GPU_C 13

/*---------------------------------------------------------------------------*/
#ifdef __cplusplus
extern "C"
{
#endif
    typedef struct rng_state_gpu
    {
        int n;
        unsigned int *state_host,*state_device;
    } rng_state_gpu;
#ifdef __cplusplus
}
#endif

/*---------------------------------------------------------------------------*/
#ifdef __cplusplus
extern "C"
{
#endif
    rng_state_gpu *allocate_rng_state_gpu(int n);
    void free_rng_state_gpu(rng_state_gpu *rng_gpu);
    void initialize_rng_state_gpu(rng_state_gpu *rng_gpu,unsigned int seed);
    void copy_rng_state_gpu_host_to_device(rng_state_gpu *rng_gpu);
    void copy_rng_state_gpu_device_to_host(rng_state_gpu *rng_gpu);
#ifdef __cplusplus
}
#endif

/*---------------------------------------------------------------------------*/
#endif

/*---------------------------------------------------------------------------*/
