/*---------------------------------------------------------------------------*/

/*!  \file  springs.h
     \brief  definitions and subroutines for slip-springs.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2009 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#ifndef INCLUDE_SPRINGS_H
#define INCLUDE_SPRINGS_H

/*---------------------------------------------------------------------------*/
#include "rng.h"
#include "chains.h"

/*---------------------------------------------------------------------------*/
#define NO_BEAD_INDEX (-1)

/*---------------------------------------------------------------------------*/
#ifdef __cplusplus
extern "C"
{
#endif
    typedef struct slip_springs
    {
        int m,z_max;
        float n_s,n0;
        float *a_host,*a_device;
        int *s_host,*s_device;
    } slip_springs;

    slip_springs *allocate_slip_springs(int m,int z_max);
    void free_slip_springs(slip_springs *springs);
    void initialize_slip_springs(slip_springs *springs,polymer_chains *chains,
                                 rng_state *rng,float n_s,float n0);
    int calculate_estimate_for_z_max(int n,float n0);
    void copy_slip_springs_host_to_device(slip_springs *springs);
    void copy_slip_springs_device_to_host(slip_springs *springs);
#ifdef __cplusplus
}
#endif

/*---------------------------------------------------------------------------*/
#endif

/*---------------------------------------------------------------------------*/
