/*---------------------------------------------------------------------------*/

/*!  \file  chain.h
     \brief  definitions and subroutines for a linear polymer chain.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2009 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#ifndef INCLUDE_CHAIN_H
#define INCLUDE_CHAIN_H

#include "vector.h"
#include "rng.h"

/*---------------------------------------------------------------------------*/
typedef struct polymer_chian
{
    int n;
    int n_b;
    double zeta_b_inv[2];
    vector3 *r;
    vector3 *r_star;
    vector3 *f;
    vector3 *f_star;
    vector3 *w;
} polymer_chain;

/*---------------------------------------------------------------------------*/
polymer_chain *allocate_polymer_chain(int n);
void free_polymer_chain(polymer_chain *chain);
void initialize_polymer_chain(polymer_chain *chain,rng_state *rng,int n_b,
                              double *zeta_b);
void calculate_stress_tensor(polymer_chain *chain,symmetric_tensor3 sigma);
void calculate_average_stress_tensor(polymer_chain **chain,int m,
                                     symmetric_tensor3 sigma);
void calculate_center_of_mass(polymer_chain *chain,vector3 rc);
void calculate_inertia_tensor(polymer_chain *chain,symmetric_tensor3 g);
void calculate_average_inertia_tensor(polymer_chain **chain,int m,
                                      symmetric_tensor3 g);
void calculate_maximum_distances(polymer_chain *chain,vector3 l_max);
void calculate_average_maximum_distances(polymer_chain **chain,int m,
                                         vector3 l_max);
void calculate_end_to_end_vector(polymer_chain *chain,vector3 re);
void calculate_average_end_to_end_vector(polymer_chain **chain,int m,
                                         vector3 re);

/*---------------------------------------------------------------------------*/
#endif

/*---------------------------------------------------------------------------*/
